@@ -133,15 +133,15 @@ a temporary convenience function __wrap_PARI_macro():
 but keep in mind that the generated this way wrapper is also not designed
 to be quick.)
 
-With C<TRUE> optional second argument, it would return an unevaluated array
+With the optional second argument C<'quote'>, it would return an unevaluated array
 of strings instead of the result of evaluation.   Special strings C<\q> etc. are
-replaced by references to appropriate (undocumented) Perl arrays.
+replaced by references to appropriate (undocumented) Perl subroutines.
 
-With C<TRUE> optional third argument, would "echo" the commands (preceded by
+With the optional third argument C<'echo'>, would "echo" the commands (preceded by
 C<"? ">) before execution.  With C<TRUE>
 optional fourth argument (command counter), would "echo" the result too
 (preceded by C<"%C = ">, with C<C> being the command counter, which is
-incremented).
+incremented).  
 
 =item C<use> with arguments
 
@@ -940,13 +940,17 @@ Ilya Zakharevich, I<ilyaz@cpan.org>
 
 =cut
 
+use strict;
+
 # $Id: Pari.pm,v 1.3 1994/11/25 23:40:52 ilya Exp ilya $
 package Math::Pari::Arr;
 
 #sub TIEARRAY { $_[0] }
 sub STORE { die "Storing into array elements unsupported" }
 
-package Math::Pari;
+package Math::Pari;		# %converted $initmem $initprimes inspected by xs
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT $AUTOLOAD
+	    %converted $initmem $initprimes %names %sections);	# XXX may need to access %names %sections?
 
 require Exporter;
 require DynaLoader;
@@ -1023,7 +1027,7 @@ sub _shiftr {
 $initmem ||= 4000000;		# How much memory for the stack
 $initprimes ||= 500000;		# Calculate primes up to this number
 
-$VERSION = '2.030519';
+$VERSION = '2.030520';
 
 my $true = 1;
 # Propagate sv_true, sv_false to SvIOK:
@@ -1088,6 +1092,7 @@ sub AUTOLOAD {
 #  goto &$cv;
 #  goto &$AUTOLOAD;
 #  &$cv;
+  no strict 'refs';
   &$1;
 #  &$AUTOLOAD;
 }
@@ -1170,6 +1175,7 @@ if (pari_version_exp() < 2009000) {
 #  *listPari = sub ($) {_listPari(old2newsec(shift))};
 }
 
+# buch* not appearing in newer PARIs (at least as of 2.3.5)
 @converted{qw(buchimag buchreal 
     buchgen buchgenforcefu buchgenfu buchinit buchinitforcefu buchinitfu 
     plotstring addhelp kill)} = (1) x 100;
@@ -1205,13 +1211,15 @@ sub _hex_cvt {
   die "Cannot hex '$in'" if length $in;
   return $ret;
 }
-%overloaded_const = ( 'int' => \&_cvt, float => \&_cvt, 'hex' => \&_hex_cvt);
-%overloaded_const_word
+my %overloaded_const = ( 'int' => \&_cvt, float => \&_cvt, 'hex' => \&_hex_cvt);
+my %overloaded_const_word
   = ( 'int' => 'integer', float => 'float', 'hex' => 'binary');
 
+my %export_ok;
+
 sub import {
   my $p=shift;
-  my @consts;			# Need to do it outside any block!
+  my @const;			# Need to do it outside any block!
   @_ = map {
     if (/^:(?!DEFAULT)(.*)/) {
       my $tag = $1;
@@ -1290,9 +1298,10 @@ for $name (keys %converted) {
   push @EXPORT_OK, $name;
   next if defined &$name;
   # string needs to format numbers to 8.3...
+  no strict 'refs';
   if ($name eq 'addhelp' or $name eq 'plotstring') {
     *$name = sub { PARI ( qq($name($_[0],"$_[1]")) ) }
-  } else {
+  } else {				# probably `kill' only
     *$name = sub { local $"=','; PARI("$name(@_)") }	# "
   }
 }
@@ -1305,16 +1314,42 @@ my %supported_cmd = qw(q 0   p setprecision   e __my_NOP);		# ???  WRONG !!!
 my $supported_cmd_rx = '(?:' . join( '|', keys %supported_cmd) . ')';
 $supported_cmd_rx = qr($supported_cmd_rx);
 my $matched_par;
-$matched_par = qr[[^()]*(?:\((??{$matched_par})\)[^()]*)*];
+$matched_par = qr[[^()]*(?:\((??{$matched_par})\)[^()]*)*];		# arbitrary string with ( and ) matching
 
 sub remove_nl ($) { (my $in = shift) =~ s/\n//g; $in }
-sub allocatemem_prot ($) {my $mem = shift;  eval {allocatemem $mem; 1} or $@ =~ /^PARI:\s\sat\s+\S*\s+line\s+\d+\.?\s*$/ or die "allocatemem died: $@"}
 
-sub parse_as_gp ($;$$$) {
-  my($in, $quote, $echo, $c) = (shift, shift, shift, shift);
+my %POSTF = qw(K 1 M 2 G 3 T 4);
+sub allocatemem_prot ($) {
+  my $mem = shift;
+  $mem =~ s/^(.*)([KMGT])/$1*1000**$POSTF{$2}/e;
+  eval {allocatemem($mem); 1} or $@ =~ /^PARI:\s\sat\s+\S*\s+line\s+\d+\.?\s*$/ or die "allocatemem($mem) died: $@"
+}
+
+sub MP___a___($) { $Math::Pari::__args::a->[shift] }
+sub __wrap_PARI_macro ($) {
+  my $name = shift;
+  sub {
+    local $Math::Pari::__args::a = [@_];
+    PARI("$name(" . (join ',', map "MP___a___($_)", 0..$#_) . ")")
+  }
+}
+
+sub PARI_with_default_mem ($) {
+  my($in) = shift;
+  $in =~ s/(?:^|;)default\(parisize,("?)($matched_par)\1\)\s*$// or return PARI($in);
+  my $s = "$2";
+  PARI($in);
+  allocatemem_prot($s);
+}
+
+sub parse_as_gp ($;$$$) {	# string, quote-and-return, <ignored>, count to use on the first numbered output (like %33 = 2*x+1)
+  my($in, $quote, $echo, $c, $def) = (shift, shift, shift, shift);
+  $def = ($quote and ($quote eq 'define' or ref $quote) and $quote);
+  $def = sub ($) {shift} if $def and not ref $def;
+  $quote = ($quote and not $def);
   $in =~ s/(\\(?!\\)|\/(?!\*)|[^"\s\\\/]|\n|"([^"\\]|\\.)*")|\/\*.*?\*\/|[^\S\n\/]+|\\\\[^\n]*/ defined($1) ? $1 : '' /ges;
   # Now all unneeded whitespace (except LF) and comments are removed
-  $in =~ s/^\{(.*?)\}$/ remove_nl $1 /gems;
+  $in =~ s/(?:^\s*\{|\{\s*$)(.*?)(?:^\s*\}|\}\s*$)/ remove_nl $1 /gems;	# XXXX In fact, braces may appear everywhere???  Strings?
   $in =~ s/(?<=[-=+*\/%^><|&,\(\[])\n(?<!--\n|\+\+\n)//gsm;	# not \ !
   $in =~ s/\\\n//g;
   $in =~ s/\n{2,}/\n/g;		# Empty lines in the middle
@@ -1328,25 +1363,21 @@ sub parse_as_gp ($;$$$) {
   return @in if $quote or not @in;
   $in = pop @in;
   my $v;
+###  	warn "  Doing($def,$quote): last -> $in";
  LOOP: {
+		  no strict 'refs';	# for subroutine names used in $foo->(), and defines if $def
     for my $IN (@in) {
-      print ref $IN ? "? $IN->[2]\n" : "? $IN\n";
-      ref $IN ? ($IN->[0] ? $IN->[0]->(PARI($IN->[1])) : last LOOP)
-            : (($c and not $IN =~ /;$/) ? (is_gnil($v = PARI($IN)) or print("%$c = $v\n"), $c++ ) : PARI($IN));	# Void context (ignored???)
+      print +($echo and ref $IN) ? "? $IN->[2]\n" : "? $IN\n";
+      ref $IN ? ($IN->[0] ? $IN->[0]->(PARI_with_default_mem($IN->[1])) : last LOOP)
+            : (($c and not $IN =~ /;$/) ? (is_gnil(PARI_with_default_mem($IN)) or print("%$c = $v\n"), $c++ ) : PARI_with_default_mem($IN));	# Void context (ignored???)
+      *{$def->("$1")} = __wrap_PARI_macro "$1" if $def and not ref $IN and $IN =~ /^(\w+)\((??{$matched_par})\)=(?!=)/;
     }
-    print ref $in ? "? $in->[2]\n" : "? $in\n";
-    ref $in ? ($in->[0] ? $in->[0]->(PARI($in->[1])) : last LOOP)
-            : (($c and not $in =~ /;$/) ? ((is_gnil($v = PARI($in)) or print("%$c = $v\n"), $c++), $v ) : PARI($in))
-  }
-}
-
-sub MP___a___($) { $Math::Pari::__args::a->[shift] }
-sub __wrap_PARI_macro ($) {
-  my $name = shift;
-  sub {
-    local $Math::Pari::__args::a = [@_];
-    PARI("$name(" . (join ',', map "MP___a___($_)", 0..$#_) . ")")
+    print +($echo and ref $in) ? "? $in->[2]\n" : "? $in\n";
+    ref $in ? ($in->[0] ? $in->[0]->(PARI_with_default_mem($in->[1])) : last LOOP)
+            : (($c and not $in =~ /;$/) ? ((is_gnil($v = PARI_with_default_mem($in)) or print("%$c = $v\n"), $c++)) : ($v = PARI_with_default_mem($in)));
+    *{$def->("$1")} = __wrap_PARI_macro "$1" if $def and not ref $in and $in =~ /^(\w+)\((??{$matched_par})\)=(?!=)/;
   }
+  $v;
 }
 
 sub link_gnuplot {
